#!/usr/bin/python
# -*- coding: UTF-8 -*-


class Dependencies(object):

    def __init__(self):
        try:
            import nibabel
            self.nibabel = nibabel
        except:
            self.nibabel = None
        try:
            import dicom
            self.dicom = dicom
        except:
            self.dicom = None


    def hasDependency(self, libname):
        return getattr(self, libname)

    



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os
from niprov.filesystem import Filesystem
from niprov.commandline import Commandline
from niprov.filefilter import FileFilter
from niprov.inspection import inspect
from niprov.jsonfile import JsonFile


def discover(root, filefilter=FileFilter(), filesys=Filesystem(), 
        listener=Commandline(), repository=JsonFile()):
    discovered = []
    dirs = filesys.walk(root)
    for (root, sdirs, files) in dirs:
        for filename in files:
            filepath = os.path.join(root, filename)
            if filefilter.include(filename):
                provenance = inspect(filepath)
                if provenance is not None:
                    discovered.append(provenance)
                    listener.fileFound(filename, provenance) 
    repository.store(discovered)   



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os
from datetime import datetime
from niprov.commandline import Commandline
from niprov.dependencies import Dependencies

formats = {'.PAR':'nibabel',
           '.dcm':'dicom'}


def inspect(fpath, listener=Commandline(), libs=Dependencies()):
    provenance = {}
    extension = os.path.splitext(fpath)[1]
    if not extension in formats:
        errmsg = 'Image file with extension {0} not supported.'
        raise ValueError(errmsg.format(extension))
    if not libs.hasDependency(formats[extension]):
        listener.missingDependencyForImage(formats[extension], fpath)
        return None
    if formats[extension] == 'nibabel':
        try:
            img = libs.nibabel.load(fpath)
        except:
            listener.fileError(fpath)
            return None
        provenance['subject'] = img.header.general_info['patient_name']
        provenance['protocol'] = img.header.general_info['protocol_name']
        acqstring = img.header.general_info['exam_date']
        dateformat = '%Y.%m.%d / %H:%M:%S'
        provenance['acquired'] = datetime.strptime(acqstring,dateformat)
    if formats[extension] == 'dicom':
        try:
            img = libs.dicom.read_file(fpath)
        except:
            listener.fileError(fpath)
            return None
        provenance['subject'] = img.PatientID
        provenance['protocol'] = img.SeriesDescription
        acqstring = img.AcquisitionDateTime.split('.')[0]
        dateformat = '%Y%m%d%H%M%S'
        provenance['acquired'] = datetime.strptime(acqstring,dateformat)
    return provenance


    



import os


class Filesystem():

    def walk(self, path):
        return os.walk(path)

    def readlines(self, path):
        with open(path) as fhandle:
            lines = fhandle.read().splitlines()
        return lines



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.jsonfile import JsonFile
from niprov.exporters import ExportFactory


def report(format=None, forFile=None, forSubject=None, repository=JsonFile(), 
        exportFactory=ExportFactory()):
    exporter = exportFactory.createExporter(format)
    if forFile:
        provenance = repository.byPath(forFile)
    elif forSubject:
        provenance = repository.bySubject(forSubject)
    else:
        provenance = repository.all()
        exporter.exportList(provenance)
    return provenance



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os
from niprov.filesystem import Filesystem


packageroot = os.path.split(os.path.split(__file__)[0])[0]
defaultfilterfile = os.path.join(packageroot,'discovery-filter.txt')

class FileFilter(object):

    def __init__(self, filesys=Filesystem()):
        self.filters = filesys.readlines(defaultfilterfile)

    def include(self, filepath):
        for filt in self.filters:
            if filt in filepath:
                return True
        return False



import unittest
from mock import Mock

class ReportingTests(unittest.TestCase):

    def test_Without_specifics_returns_all_files(self):
        import niprov
        log = Mock()
        repo = Mock()
        out = niprov.report(repository=repo, exportFactory=Mock())
        self.assertEqual(out, repo.all())

    def test_Can_report_on_one_file_specifically(self):
        import niprov
        log = Mock()
        repo = Mock()
        repo.byPath.side_effect = lambda x: ('prov for', x)
        out = niprov.report(forFile='afile.f', repository=repo, 
            exportFactory=Mock())
        self.assertEqual(out, repo.byPath('afile.f'))

    def test_Can_report_on_all_files_for_subject(self):
        import niprov
        log = Mock()
        repo = Mock()
        repo.bySubject.side_effect = lambda x: ('prov for', x)
        out = niprov.report(forSubject='Jane Doe', repository=repo, 
            exportFactory=Mock())
        self.assertEqual(out, repo.bySubject('Jane Doe'))

    def test_Obtains_exporter_from_factory_and_passes_it_provenance(self):
        import niprov
        factory = Mock()
        repo = Mock()
        niprov.report(repository=repo, exportFactory=factory)
        factory.createExporter.assert_any_call(None)
        niprov.report(format='html', repository=repo, exportFactory=factory)
        factory.createExporter.assert_any_call('html')

    def test_Passes_provenance_to_exporter(self):
        import niprov
        factory = Mock()
        repo = Mock()
        niprov.report(repository=repo, exportFactory=factory)
        factory.createExporter().exportList.assert_any_call(repo.all())






import unittest
from mock import Mock
from datetime import datetime

class InspectionTests(unittest.TestCase):

    def test_Loads_file_with_nibabel(self):
        import niprov.inspection
        log = Mock()
        libs = self.setupNibabel()
        libs.hasDependency.return_value = True
        niprov.inspection.inspect('/p/f1.PAR', listener=log, libs=libs)
        libs.nibabel.load.assert_any_call('/p/f1.PAR')

    def test_Doesnt_use_nibabel_if_not_installed(self):
        import niprov.inspection
        log = Mock()
        libs = self.setupNibabel()
        libs.hasDependency.return_value = False
        provenance = niprov.inspection.inspect('/p/f1.PAR', listener=log, libs=libs)
        self.assertRaises(AssertionError,
            libs.nibabel.load.assert_any_call, '/p/f1.PAR')

    def test_If_dcm_passed_uses_pydicom_to_open(self):
        import niprov.inspection
        log = Mock()
        libs = self.setupPydicom()
        libs.hasDependency.return_value = True
        provenance = niprov.inspection.inspect('/p/f1.dcm', listener=log, libs=libs)
        # doesnt complain about missing pydicom        
        self.assertRaises(AssertionError,
            log.missingDependencyForImage.assert_any_call, 'dicom','/p/f1.dcm')
        # uses pydicom
        libs.dicom.read_file.assert_any_call('/p/f1.dcm')

    def test_If_dcm_passed_but_pydicom_not_installed_tells_listener(self):
        import niprov.inspection
        log = Mock()
        libs = self.setupPydicom()
        libs.hasDependency.return_value = False
        provenance = niprov.inspection.inspect('/p/f1.dcm', listener=log, libs=libs)
        self.assertRaises(AssertionError,
            libs.dicom.read_file.assert_any_call, '/p/f1.dcm')
        log.missingDependencyForImage.assert_any_call('dicom','/p/f1.dcm')

    def test_If_nothing_inspected_returns_None(self):
        import niprov.inspection
        log = Mock()
        libs = self.setupNibabel()
        libs.hasDependency.return_value = False
        provenance = niprov.inspection.inspect('/p/f1.PAR', listener=log, libs=libs)
        self.assertIsNone(provenance)

    def test_Gets_basic_info_from_nibabel_and_returns_it(self):
        import niprov.inspection
        log = Mock()
        libs = self.setupNibabel()
        libs.hasDependency.return_value = True
        out = niprov.inspection.inspect('/p/f1.PAR', listener=log, libs=libs)
        self.assertEqual(out['subject'], 'John Doeish')
        self.assertEqual(out['protocol'], 'T1 SENSE')
        self.assertEqual(out['acquired'], datetime(2014, 8, 5, 11, 27, 34))

    def test_Gets_basic_info_from_pydicom_and_returns_it(self):
        import niprov.inspection
        log = Mock()
        libs = self.setupPydicom()
        libs.hasDependency.return_value = True
        out = niprov.inspection.inspect('/p/f1.dcm', listener=log, libs=libs)
        self.assertEqual(out['subject'], 'John Doeish')
        self.assertEqual(out['protocol'], 'T1 SENSE')
        self.assertEqual(out['acquired'], datetime(2014, 8, 5, 12, 19, 14))

    def test_If_error_during_inspection_tells_listener_and_returns_None(self):
        import niprov.inspection
        log = Mock()
        libs = Mock()
        libs.hasDependency.return_value = True
        libs.nibabel.load.side_effect = ValueError
        out = niprov.inspection.inspect('/p/f1.PAR', listener=log, libs=libs)
        self.assertIsNone(out)
        log.fileError.assert_any_call('/p/f1.PAR')


    def setupNibabel(self):
        libs = Mock()
        img = Mock()
        img.header.general_info = {
            'exam_date':'2014.08.05 / 11:27:34',
            'protocol_name':'T1 SENSE',
            'patient_name':'John Doeish'}
        libs.nibabel.load.return_value = img
        return libs

    def setupPydicom(self):
        libs = Mock()
        img = Mock()
        img.AcquisitionDateTime = '20140805121914.59000'
        img.SeriesDescription = 'T1 SENSE'
        img.PatientID = 'John Doeish'
        libs.dicom.read_file.return_value = img
        return libs        




#!/usr/bin/python
# -*- coding: UTF-8 -*-


class Commandline(object):

    def fileFound(self, fname, provenance):
        template = '{1[acquired]} {1[subject]} {1[protocol]:24} {0}'
        print(template.format(fname,provenance))

    def missingDependencyForImage(self, lib, fpath):
        template = 'Missing python package "{0}" to read file: {1}'        
        print(template.format(lib, fpath))

    def fileError(self, fpath):
        print('Error opening file: {0}'.format(fpath))
    



import unittest
from mock import Mock
import os.path as ospath

class DiscoveryTests(unittest.TestCase):

    def test_Tells_listener_about_files_found(self):
        import niprov.discovery
        niprov.discovery.inspect = Mock()
        log = Mock()
        filt = self.setupFilter('.x')
        os = Mock()
        os.walk.return_value = [('root',[],['/p/f1.x','/p/f2.x']),
            ('root',[],['/p/p2/f3.x'])] #(dirpath, dirnames, filenames)
        niprov.discovery.discover('root', filesys=os, listener=log, 
            filefilter=filt, repository=Mock())
        os.walk.assert_called_with('root')
        log.fileFound.assert_any_call('/p/f1.x', niprov.discovery.inspect())
        log.fileFound.assert_any_call('/p/f2.x', niprov.discovery.inspect())
        log.fileFound.assert_any_call('/p/p2/f3.x', niprov.discovery.inspect())

    def test_file_filters(self):
        import niprov.discovery
        niprov.discovery.inspect = Mock()
        log = Mock()
        os = Mock()
        filt = self.setupFilter('valid.file')
        os.walk.return_value = [('root',[],['valid.file','other.file'])]
        niprov.discovery.discover('root', filesys=os, listener=log,
            filefilter=filt, repository=Mock())
        log.fileFound.assert_any_call('valid.file', niprov.discovery.inspect())
        self.assertRaises(AssertionError,
            log.fileFound.assert_any_call,'other.file', niprov.discovery.inspect())

    def test_Calls_inspect(self):
        import niprov.discovery
        niprov.discovery.inspect = Mock()
        log = Mock()
        os = Mock()
        filt = self.setupFilter('.valid')
        os.walk.return_value = [('root',[],['a.valid','other.file','b.valid'])]
        niprov.discovery.discover('root', filesys=os, listener=log,
            filefilter=filt, repository=Mock())
        niprov.discovery.inspect.assert_any_call(ospath.join('root','a.valid'))
        niprov.discovery.inspect.assert_any_call(ospath.join('root','b.valid'))

    def test_If_inspect_returns_no_provenance_dont_call_fileFound(self):
        import niprov.discovery
        niprov.discovery.inspect = Mock()
        niprov.discovery.inspect.return_value = None
        log = Mock()
        os = Mock()
        filt = self.setupFilter('valid.file')
        os.walk.return_value = [('root',[],['valid.file','other.file'])]
        niprov.discovery.discover('root', filesys=os, listener=log,
            filefilter=filt, repository=Mock())
        self.assertRaises(AssertionError,
            log.fileFound.assert_any_call,'valid.file', niprov.discovery.inspect())

    def test_Hands_provenance_to_repository(self):
        import niprov.discovery
        niprov.discovery.inspect = Mock()
        niprov.discovery.inspect.side_effect = lambda x: ('p', x)
        os = Mock()
        repo = Mock()
        filt = self.setupFilter('.valid')
        os.walk.return_value = [('root',[],['a.valid','other.file','b.valid'])]
        niprov.discovery.discover('root', filesys=os, listener=Mock(),
            filefilter=filt, repository=repo)
        repo.store.assert_any_call([('p', 'root/a.valid'), ('p', 'root/b.valid')])



    def setupFilter(self, valid):
        def filter_side_effect(*args):
            if valid in args[0]:
                return True
            return False
        filt = Mock()
        filt.include = Mock(side_effect=filter_side_effect)
        return filt
        




import json


class JsonFile(object):

    def store(self, provenance):
        with open('provenance.json', 'w') as fp:
            json.dump(provenance, fp, default=datetime2json)

    def all(self):
        with open('provenance.json', 'r') as fp:
            provenance = json.load(fp)
        return provenance

    def byPath(self, path):
        all = self.all()
        return []

    def bySubject(self, subject):
        all = self.all()
        return [f for f in all if f['subject']==subject]

def datetime2json(obj):
    return obj.isoformat() if hasattr(obj, 'isoformat') else obj



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from setuptools import setup, find_packages


setup(name='niprov',
      version='0.0',
      author='Jasper J.F. van den Bosch',
      author_email='japsai@gmail.com',
      description='provenance for neuroimaging data',
      packages=find_packages(),
      url = 'https://github.com/ilogue/niprov',
      test_suite="tests",
      scripts=['executables/discover', 'executables/provenance'],
      zip_safe=False,
      classifiers=[
            'License :: OSI Approved :: BSD License',
            'Intended Audience :: Science/Research',
            'Topic :: Scientific/Engineering']
      )



import unittest
from mock import Mock
import os




class FileFilterTests(unittest.TestCase):

    def test_reads_extensions_from_default_file(self):
        packageroot = os.path.split(os.path.split(__file__)[0])[0]
        defaultfilterfile = os.path.join(packageroot,'discovery-filter.txt')
        from niprov.filefilter import FileFilter
        filesys = Mock()
        filesys.readlines.return_value = ['.abc','.def']
        filt = FileFilter(filesys=filesys)
        filesys.readlines.assert_called_with(defaultfilterfile)
        self.assertTrue(filt.include('/p/sth.abc'))
        self.assertTrue(filt.include('/p/sth.def'))
        self.assertFalse(filt.include('/p/sth.xyz'))


